"""comment.py: Hog 4 comment macro interpreter and OSC bridge."""

import configparser
import logging

from antlr4 import CommonTokenStream, InputStream, ParseTreeWalker
from antlr4.error.ErrorListener import ErrorListener
from pythonosc import udp_client

from .commentmacro.CommentMacroLexer import CommentMacroLexer
from .commentmacro.CommentMacroParser import CommentMacroParser
from .OscListener import OscCommentMacroListener


class SyntaxErrorListener(ErrorListener):
    """An error listener that raises SyntaxError exceptions."""

    def syntaxError(self, recognizer, offendingSymbol, line, column, msg, e):
        raise SyntaxError("line "+str(line)+":"+str(column)+" "+msg)


def load_config(file='server.cfg'):
    """Load an ini style configuration file."""
    # empty server dictionary
    servers = {}

    # open config file
    config = configparser.ConfigParser(allow_no_value=True)
    config.read(file)

    # set up each hog device
    for name in config.get('network', 'hogs').split(','):
        try:
            # move to config section
            server = config[name.strip().strip('\"')]
            # read settings
            addr = server.get("ip", "10.0.0.100")
            port = server.getint("port", 7001)
            net = server.getint("net", 1)
            # osc clients are added to the dictionary with the net # as the key
            logging.info("Adding Hog device at net# %d", str(net))
            servers[net] = udp_client.SimpleUDPClient(addr, port)
        except KeyError as exception:
            print('Error configuring net#', net, exception)
            continue
    return servers


# init reusable walker and listener
WALKER = ParseTreeWalker()
LISTENER = OscCommentMacroListener()


def comment(text):
    """Process comment macro input."""
    # force upper case
    text = text.upper()
    # generate text stream
    input_stream = InputStream(text)
    # lex the text stream
    lexer = CommentMacroLexer(input_stream)
    # get token stream from lexer
    stream = CommonTokenStream(lexer)
    # parse the token steam
    parser = CommentMacroParser(stream)
    # attach an error handler to the parser
    parser._listeners.append(SyntaxErrorListener())
    try:
        # get tree from parser
        tree = parser.prog()
        # walk the tree
        WALKER.walk(LISTENER, tree)
    except SyntaxError as exception:
        logging.debug(exception)     # antlr internal listener prints the error
#                                    # log it to the debug logging anyway
